@@ -19,7 +19,7 @@ return 1;
 sub free_space
 {
 local($out);
-$out = `df -t $_[0]`;
+$out = &backquote_command("df -t ".quotemeta($_[0]));
 $out =~ /(\d+) blocks\s+(\d+) files\n.*\s+(\d+) blocks\s+(\d+) files/;
 return ($3, $1, $4, $2);
 }
@@ -52,13 +52,14 @@ return $_[0]->[3] =~ /,quota/ || $_[0]->[3] =~ /^quota/ ? 1 : 0;
 sub filesystem_users
 {
 local($rep, @rep, $n, %hasu, $u);
-$rep = &backquote_logged("$config{'user_repquota_command'} $_[0] 2>&1");
+$rep = &backquote_command("$config{'user_repquota_command'} ".
+			  quotemeta($_[0])." 2>&1");
 if ($?) { return -1; }
 setpwent();
 while(@uinfo = getpwent()) {
 	$hasu{$uinfo[0]}++;
 	}
-endpwent();
+endpwent() if ($gconfig{'os_type'} ne 'hpux');
 @rep = split(/\n/, $rep); @rep = @rep[3..$#rep];
 for($n=0; $n<@rep; $n++) {
 	if ($rep[$n] =~ /(\S+)\s+..\s+(\d+)\s+(\d+)\s+(\d+).*(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+..(.{7})(.{7})(.{7}).{13}(.{7})(.{7})(.{7})/) {
@@ -105,13 +106,16 @@ return $rv;
 # be 1 for user only, 2 for group only or 3 for user and group
 sub quotaon
 {
+return if (&is_readonly_mode());
 local($qf, $out);
 $qf = "$_[0]/quotas";
 if (!(-r $qf)) {
-	open(QUOTAFILE, "> $qf"); close(QUOTAFILE);
-	chmod(0600, $qf);
+	&open_tempfile(QUOTAFILE, ">$qf", 0, 1);
+	&close_tempfile(QUOTAFILE);
+	&set_ownership_permissions(undef, undef, 0600, $qf);
 	}
-$out = &backquote_logged("$config{'user_quotaon_command'} $_[0] 2>&1");
+$out = &backquote_logged("$config{'user_quotaon_command'} ".
+			 quotemeta($_[0])." 2>&1");
 if ($?) { return $out; }
 return undef;
 }
@@ -120,8 +124,10 @@ return undef;
 # Turn off quotas for some filesystem
 sub quotaoff
 {
+return if (&is_readonly_mode());
 local($out);
-$out = &backquote_logged("$config{'user_quotaoff_command'} $_[0] 2>&1");
+$out = &backquote_logged("$config{'user_quotaoff_command'} ".
+			 quotemeta($_[0])." 2>&1");
 if ($?) { return $out; }
 return undef;
 }
@@ -130,7 +136,8 @@ return undef;
 # Runs quotacheck on some filesystem
 sub quotacheck
 {
-$out = &backquote_logged("$config{'quotacheck_command'} $_[0] 2>&1");
+$out = &backquote_logged("$config{'quotacheck_command'} ".quotemeta($_[0]).
+			 " 2>&1");
 if ($?) { return $out; }
 return undef;
 }
@@ -140,7 +147,8 @@ return undef;
 sub copy_user_quota
 {
 for($i=1; $i<@_; $i++) {
-	$out = &backquote_logged("$config{'user_copy_command'} $_[0] $_[$i] 2>&1");
+	$out = &backquote_logged("$config{'user_copy_command'} ".
+				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
 	if ($?) { return $out; }
 	}
 return undef;
@@ -152,7 +160,7 @@ return undef;
 sub user_filesystems
 {
 local($n, $_);
-open(QUOTA, "$config{'user_quota_command'} $_[0] |");
+open(QUOTA, "$config{'user_quota_command'} ".quotemeta($_[0])." |");
 $n=0; while(<QUOTA>) {
 	chop;
 	if (/^(Disk|Filesystem)/) { next; }
